;###########################################################################
;### Autor   : Jörn Kierstein                                            ###
;### Erstellt: 04.01.2014                                                ###
;### IDE     : C64 Studio 5.3                                            ###
;###                                                                     ###
;### (c) 2014, 2018 by www.retro-programming.de                          ###
;###########################################################################

!to "landung.prg",cbm

;*** Konstanten einbinden
!source "..\_Includes_\system_const.asm"
!source "..\_Includes_\love_const.asm"

RASTERIRQ_SKY       = $00           ;ab hier die Himmelfarbe
RASTERIRQ_LASER     = $30           ;        das 'Laser-Raster'
RASTERIRQ_CANYON    = $32           ;        die Canyon-Farbe
SPACESHIP_STARTX    = $a5           ;X-Position und
SPACESHIP_STARTY    = $32           ;Y-Position, an der das Schiff startet


!zone BASIC_Start
;*** Startadresse
*=$0801

;*** BASIC-Zeile
 !word main-2
 !word 2018
 !text $9E," 2062",$00,$00,$00

!zone Main
;*******************************************************************************
;*** Einstieg ins Assemblerprogramm
;*******************************************************************************
main
 jsr initLander                     ;Level starten 

;*** Raster-IRQ einrichten
 sei                                ;IRQs sperren
 lda #<landerIRQ                    ;Routine unserer Rasten-Routine
 sta JT_IRQVECTOR                   ;im den RAM-Vector speichern
 lda #>landerIRQ                    ;auch das MSB
 sta JT_IRQVECTOR+1
 lda VIC_IRQMASK                    ;Raster-IRQs auslösen soll
 ora #%00000001                     ;Dem VIC-II mitteilen, dass er 
 sta VIC_IRQMASK                    ;Raster-IRQs auslösen soll
 lda #RASTERIRQ_SKY                 ;Erster Raster-IRQ für den Himmel
 sta VIC_RASTERROWPOS                         
 lda VIC_CONTROLREG1                ;Höchstes BIT für die Rasterzeile
 and #%01111111                     ;löschen
 sta VIC_CONTROLREG1
 cli                                ;IRQs wieder erlauben

gameloop
 jmp gameloop                       ;Hauptschleife



!zone initLander
;*******************************************************************************
;*** Leveldaten zurücksetzen
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, X, Y, SR
;*******************************************************************************
initLander
 lda #SPACESHIP_STARTX              ;Startpositione setzen
 sta SpaceshipPos                   ;X
 lda #SPACESHIP_STARTY              ;und
 sta SpaceshipPos+1                 ;Y
 jsr initSprites                    ;Sprites initialisieren
 jsr drawCanyon
 rts



!zone initSprites
;*******************************************************************************
;*** Sprites initialisieren
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, X, SR
;*******************************************************************************
initSprites
 ldx #spriteSpaceship/64            ;Nr. des ersten 64-Byte-Sprite-Blocks
 stx SPRITEPOINTER0+4               ;Spritedaten zuweisen
 inx
 stx SPRITEPOINTER0+0
 inx
 stx SPRITEPOINTER0+2
 inx
 stx SPRITEPOINTER0+3
 inx
 stx SPRITEPOINTER0+1

 jsr updateSprites                  ;Sprite-Positionen setzen

 lda #COLOR_BLACK                   ;Farben der Sprites setzen
 sta VIC_SPRITE0COLOR

 lda #COLOR_LIGHTGREY
 sta VIC_SPRITE4COLOR

 lda #COLOR_RED
 sta VIC_SPRITEMULTICOLOR0
 lda #COLOR_YELLOW
 sta VIC_SPRITEMULTICOLOR1

 lda #%00001110                     ;nur die "Feuer"-Sprites sind MultiColor
 sta VIC_SPRITEMULTICOLOR

 lda #%00000000
 sta VIC_SPRITEDEEP
 sta VIC_SPRITEDOUBLEWIDTH
 sta VIC_SPRITEDOUBLEHEIGHT

 lda #%00010001                     ;erstmal nur das 'Schiff' anzeigen
 sta VIC_SPRITEACTIVE

 rts                                ;zurück



!zone updateSprites
;*******************************************************************************
;*** Sprites positionieren
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : X, SR
;*******************************************************************************
updateSprites
 ldx SpaceshipPos                   ;X-Position holen
 stx VIC_SPRITE0X                   ;für Sprite-0
 stx VIC_SPRITE1X                   ;1 und
 stx VIC_SPRITE4X                   ;4 übernehmen
 inx                                ;rechtes 'Feuer'
 inx                                ;um drei Pixel nach rechts
 inx                                ;verschieben
 stx VIC_SPRITE3X
 ldx SpaceshipPos                   ;nochmal die X-Position holen
 dex                                ;und linkes 'Feuer'
 dex                                ;um drei Pixel nach links
 dex                                ;verschieben
 stx VIC_SPRITE2X
 ldx SpaceshipPos+1                 ;Y-Position
 stx VIC_SPRITE0Y                   ;für          
 stx VIC_SPRITE1Y                   ;alle
 stx VIC_SPRITE2Y                   ;Sprites
 stx VIC_SPRITE3Y                   ;identisch
 stx VIC_SPRITE4Y         
 rts                                ;zurück



!zone drawCanyon
;*******************************************************************************
;*** Den Canyon zeichnen
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, X, Y, SR
;*******************************************************************************
drawCanyon
 lda #COLOR_ORANGE
 sta VIC_BORDERCOLOR
 lda #COLOR_DARKGREY
 sta VIC_BACKGROUNDCOLOR
 ;*** Canyon anzeigen
 ldx #<screenCanyon                 ;LSB des Canyonbildes ins X-Register
 ldy #>screenCanyon                 ;MSB des Canyonbildes ins Y-Register
 jsr drawScreen                     ;Bild ausgeben
 rts                                ;zurück



!zone landerIRQ
;*******************************************************************************
;*** Interrupthandler
;***
;*** Rasterinterrupts werden hier verarbeitet,
;*** andere Interrupts werden an die System-Routine $EA31 weitergeleitet.
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : -
;*******************************************************************************
landerIRQ
 lda VIC_IRQSTATUS                  ;prüfen ob der IRQ vom VIC-II kam
 bmi doRasterIRQ                    ;falls ja zur Raster-Routine
 lda CIA1_IRQCONTROL                ;sonst CIA-IRQ bestätigen
 cli                                ;Interrupts erlauben
 jmp ROM_IRQFUNCTION                ;und zur System-Funktion springen
doRasterIRQ
 sta VIC_IRQSTATUS                  ;VIC-IRQ bestätigen
 lda VIC_RASTERROWPOS               ;aktuelle Rasterzeile in den Akku
 bne .laser                         ;Wenn nicht 0, dann auf 'Laser' prüfen
 lda #COLOR_LIGHTBLUE               ;sonst hellblau in den Akku
 sta VIC_BORDERCOLOR                ;die Rahmenfarbe auf darauf setzen
 lda #RASTERIRQ_LASER               ;nächsten Raster-IRQ beim Laser
 sta VIC_RASTERROWPOS                
 jmp .exit                          ;und IRQ verlassen
.laser
 cmp #RASTERIRQ_LASER               ;Aktuelle Rasterzeile=Beginn des Lasers?
 bne .canyon                        ;falls nicht, weiter zum Canyon
 inc LaserFlash                     ;'Farbe' fürs Laser-Raster erhöhen,
 lda LaserFlash                     ;in den Akku holen
 sta VIC_BORDERCOLOR                ;und Rahmenfarbe damit setzen
 lda #RASTERIRQ_CANYON              ;nächsten Raster-IRQ beim Canyon
 sta VIC_RASTERROWPOS
 jmp .exit                          ;IRQ verlassen
.canyon      
 lda #COLOR_ORANGE                  ;hier beginnt auf jeden Fall der Canyon
 sta VIC_BORDERCOLOR                ;also Canyon-Farbe in den Rahmen
 lda #RASTERIRQ_SKY                 ;und nächsten Raster-IRQ für den Himmel
 sta VIC_RASTERROWPOS
.exit
 pla                                ;Register zurückholen
 tay                                ;Y
 pla
 tax                                ;X
 pla                                ;Akku

 rti                                ;IRQ verlassen



;*******************************************************************************
;*** Allgemeine Funktionen einbinden
;*******************************************************************************
!source "..\_Includes_\love_lib.asm"



!zone Data
;*******************************************************************************
;*** Daten
;*******************************************************************************
LaserFlash
 !byte $00                          ;Rahmenfarbe fürs Laser-Raster

SpaceshipPos
 !byte SPACESHIP_STARTX             ;X- und  
 !byte SPACESHIP_STARTY             ;Y-Position des Landungsschiffs



;*** Die Sprites
!align 63,0
spriteSpaceship
 !media "Ship.spriteproject",sprite,0,5

 
;*** Der Canyon
screenCanyon
 !media "Canyon.charscreen",charcolor